<?php

/*
	Plugin Name: БИФИТ Касса
	Description: Облачная фискализация чеков
	Author: БИФИТ
	Version: 1.0.5
 */

// Подключаем вспомогательные функции
require_once dirname(__FILE__) . '/includes/helpers.php';
require_once dirname(__FILE__) . '/includes/hooks.php';

new BifitKassa();

class BifitKassa
{
	private static $lang = array(); // массив с переводами плагина (локали)
	private static $pluginName = ''; // название плагина (соответствует названию папки)
	private static $path = ''; //путь до файлов плагина

	/**
	 * В конструктор класса происходит инициализация всех необходимых данных для работы плагина, таких как:
	 * -системное имя плагина;
	 * -локали;
	 * -страница настроек плагина и активация плагина;
	 * -подключение css и js для админки;
	 */
	public function __construct()
	{
		// Название плагина соответствует имени папки плагина
		self::$pluginName = PM::getFolderPlugin(__FILE__);

		// Инициализация локалей плагина
		self::createLocale();

		// Установка пути до папки плагина
		self::$path = PLUGIN_DIR . self::$pluginName;

		// Установка названия метода выполняющегося при активации плагина.
		// При активации плагина будет выполняться метод этого класса с названием: activate()
		mgActivateThisPlugin(__FILE__, array(__CLASS__, 'activate'));

		// Установка названия метода выполняющегося при открытии настроек плагина.
		// При переходе в настройки плагина будет выполняться метод этого класса с названием: pageSettingsPlugin()
		mgAddAction(__FILE__, array(__CLASS__, 'pageSettingsPlugin'));

		// Подключаемся к хуку обновления заказа Models_Order_updateOrder
		mgAddAction('Models_Order_updateOrder', 'BifitKassa_updateOrder', 1, 1);
	}

	/**
	 * Подготовка файлов локалей
	 */
	private function createLocale()
	{
		// Подключение локали плагина для текущего языка из папки /locales
		self::$lang = PM::plugLocales(self::$pluginName);

		// Подключение локали движка для текущего языка /mg-admin/locales
		include 'mg-admin/locales/' . MG::getSetting('languageLocale') . '.php';

		// Объединение локалей в один массив для дальнейшей работы
		$lang = array_merge($lang, self::$lang);

		self::$lang = $lang;
	}

	/**
	 * Метод выполняющийся при активации палагина
	 * Создает таблицу в базе данных, опции и примеры записей
	 * Используемые методы:
	 *  createDateBase()
	 *  createOptions()
	 *  insertExamples()
	 */
	static function activate()
	{
		self::createDateBase();
		self::createOptions();
	}

	/**
	 * Создает таблицу плагина в БД
	 *  id            [число]       - ключ записи
	 *  order_id      [число]       - идентификатор заказа
	 *  order_number  [число]       - номер заказа
	 *  order_date    [дата+время]  - дата заказа
	 *  action_date   [дата+время]  - дата запроса
	 *  receipt_link  [символы]     - ссылка на чек
	 *  receipt_id    [число]       - идентификатор документа
	 *  log           [текст]       - лог операции
	 *  receipt_type  [символы]     - тип документа
	 */
	private function createDateBase()
	{
		DB::query(
			"CREATE TABLE IF NOT EXISTS `" .
				PREFIX .
				self::$pluginName .
				"`(
				`id` int(11) NOT NULL AUTO_INCREMENT COMMENT 'Порядковый номер записи',
				`order_id` int(11) NOT NULL COMMENT 'Идентификатор заказа',
				`order_number` varchar(50) NOT NULL COMMENT 'Номер заказа',
				`order_date` DATETIME NOT NULL COMMENT 'Дата заказа',
				`action_date` DATETIME NOT NULL COMMENT 'Дата запроса',
				`receipt_link` varchar(255) NOT NULL COMMENT 'Ссылка на чек',
				`receipt_id` int(11) NOT NULL COMMENT 'Идентификатор документа',
				`receipt_type` varchar(50) NOT NULL COMMENT 'Тип документа',
				`log` text NOT NULL COMMENT 'лог операции',
				PRIMARY KEY (`id`)
				) ENGINE=MyISAM DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;"
		);
	}

	/**
	 * Создает опции плагина, доступные в дальнейшем в настройках плагина
	 */
	private function createOptions()
	{
		// Если опции еще не были созданы, то создать
		if (MG::getSetting(self::$pluginName . '-option') == null) {
			// Список опций
			$array = array(
				"test" => "1", // Тестовый режим
				"token" => "", // Токен
				"cashier_name" => "Иванов И.И.", // Имя кассира
				"cashier_inn" => "123456789012", // ИНН кассира
				"payment_address" => "https://site.ru", // Место расчётов
				"address_type" => "email", // Адрес клиента
				"calculation_method" => "prepay_full", // Признак способа расчёта
				"payment_type" => "cash", // Тип оплаты
				"vat_mode" => "1", // Единый НДС
				"vat" => "without_vat", // Ставка НДС
				"use_type_1" => "0", // Использовать Приход
				"use_type_2" => "0", // Использовать Возврат прихода
				"use_type_3" => "0", // Использовать Расход
				"use_type_4" => "0", // Использовать Возврат расхода
				"use_type_5" => "0", // Использовать Связь
				"order_paid_1" => "-1", // Заказ оплачен: Да(1), Нет(0), Не учитывать(-1)
				"order_paid_2" => "-1", // Заказ оплачен: Да(1), Нет(0), Не учитывать(-1)
				"order_paid_3" => "-1", // Заказ оплачен: Да(1), Нет(0), Не учитывать(-1)
				"order_paid_4" => "-1", // Заказ оплачен: Да(1), Нет(0), Не учитывать(-1)
				"order_paid_5" => "-1", // Заказ оплачен: Да(1), Нет(0), Не учитывать(-1)
				"tax_system_1" => "common", // Система налогообложения
				"tax_system_2" => "common", // Система налогообложения
				"tax_system_3" => "common", // Система налогообложения
				"tax_system_4" => "common", // Система налогообложения
				"tax_system_5" => "common", // Система налогообложения
				"pm_statuses_1" => "", // Статус(ы) оплаты
				"pm_statuses_2" => "", // Статус(ы) оплаты
				"pm_statuses_3" => "", // Статус(ы) оплаты
				"pm_statuses_4" => "", // Статус(ы) оплаты
				"pm_statuses_5" => "", // Статус(ы) оплаты
				"pm_ids_1" => "", // Способ(ы) оплаты
				"pm_ids_2" => "", // Способ(ы) оплаты
				"pm_ids_3" => "", // Способ(ы) оплаты
				"pm_ids_4" => "", // Способ(ы) оплаты
				"pm_ids_5" => "", // Способ(ы) оплаты
				"login" => "", // Логин
				"password" => "", // Пароль
				"organization" => "", // Организация
				"trade_object" => "", // Торговый объект
				"lk_courier_delivery" => "", // Доставка Курьером
				"lk_self_delivery" => "", // Доставка самовывоз
				"lk_new_courier_order_status" => "new", // Статус для нового заказа Курьером
				"lk_use_refused" => "0", // Удалять заказ в кабинете
				"lk_refused_statuses" => "" // Статус заказа для удаления
			);

			// Добавляем опции
			MG::setOption(array(
				'option' => self::$pluginName . '-option',
				'value' => addslashes(serialize($array))
			));
		}
	}

	/**
	 * Создает md5-версию файла по дате изменения
	 */
	public function createFileVersion($url)
	{
		$version = file_exists($url) ? filemtime($url) : '';

		return $version != '' ? '?' . hash("md5", $version) : '?XXX';
	}

	/**
	 * Выводит страницу настроек плагина в админке
	 *
	 * Используемые методы:
	 *  maxDate()
	 *  minDate()
	 *  getOptions()
	 *  preparePageSettings()
	 */
	static function pageSettingsPlugin()
	{
		/**
		 * Подготовка переменных локалей, настроек и имени плагина для передачи в верстку страницы настроек
		 */

		// Локали
		$lang = self::$lang;
		// Имя плагина для имени секции на странице
		$pluginName = self::$pluginName;
		// Опции плагина
		$options = self::getOptions();

		/**
		 * Подготовка переменных для создания фильтров по записям и передачи в верстку страницы настроек плагина
		 */

		// Варианты выбора типа документа
		$types = array(
			'sale' => $lang['TYPE_SALE'],
			'sale_return' => $lang['TYPE_SALE_RETURN'],
			'purchase' => $lang['TYPE_PURCHASE'],
			'purchase_return' => $lang['TYPE_PURCHASE_RETURN']
		);

		// Значения по умолчанию для фильтра по дате
		$maxDate = self::maxDate();
		$minDate = self::minDate();

		// Конструктор фильтров:
		/**
		 *  '<название поля записи>' => array(
		 *  'type'  => '<тип записи>', // текст, числа, список, промежуток и т.д.
		 *  'label' => '<подпись поля для вывода>',
		 *  'value' => '<значение для вывода (не для фильтрации)>',
		 *  //...и прочее в зависимости от типа>
		 *  )
		 */
		$property = array(
			// Параметр фильтра по идентификатору заказа
			'order_id' => array(
				'type' => 'text',
				'label' => $lang['ORDER_ID'],
				'value' => !empty($_POST['order_id'])
					? $_POST['order_id']
					: null
			),

			// Параметр фильтра по номеру заказа
			'order_number' => array(
				'type' => 'text',
				'label' => $lang['ORDER_NUMBER'],
				'value' => !empty($_POST['order_number'])
					? $_POST['order_number']
					: null
			),

			// Параметр фильтра по промежутку
			'order_date' => array(
				'type' => 'between',
				'label0' => $lang['ADD_DATE'],
				'label1' => $lang['ADD_DATE_FROM'],
				'label2' => $lang['ADD_DATE_TO'],
				'min' => !empty($_POST['order_date'][0])
					? $_POST['order_date'][0]
					: $minDate,
				'max' => !empty($_POST['order_date'][1])
					? $_POST['order_date'][1]
					: $maxDate,
				'special' => 'date',
				'class' => 'date'
			),

			// Параметр фильтра по типу документа
			'receipt_type' => array(
				'type' => 'select',
				'option' => $types,
				'selected' => !empty($_POST['receipt_type'])
					? $_POST['receipt_type']
					: 'null', // Выбранный пункт (сравнивается по значению)
				'label' => $lang['RECEIPT_TYPE']
			),

			// Обязательный скрытый параметр для сортировки по выбранному полю
			'sorter' => array(
				'type' => 'hidden', // текстовый инпут
				'label' => 'сортировка по полю',
				'value' => !empty($_POST['sorter']) ? $_POST['sorter'] : null
			)
		);

		// Если фильтр уже был применен, запомним это с помощью еще одного скрытого параметра
		if (isset($_POST['applyFilter'])) {
			$property['applyFilter'] = array(
				'type' => 'hidden', // текстовый инпут
				'label' => 'флаг примения фильтров',
				'value' => 1
			);
		}

		// Создаем фильтр по заготовленным параметрам
		$filter = new Filter($property);

		/**
		 * Подготовка значений для параметров фильтра
		 */

		// Создаем массив для значений и указываем те, которые имееют значение по умолчанию
		$arFilter = array(
			'order_id' => !empty($_POST['order_id'])
				? $_POST['order_id']
				: null,
			'order_number' => !empty($_POST['order_number'])
				? $_POST['order_number']
				: null,
			'receipt_type' => !empty($_POST['receipt_type'])
				? $_POST['receipt_type']
				: 'null',
			'order_date' => array(
				!empty($_POST['order_date'][0])
					? $_POST['order_date'][0]
					: $minDate,
				!empty($_POST['order_date'][1])
					? $_POST['order_date'][1]
					: $maxDate,
				'date'
			)
		);

		// Добавляем значения в фильтр, сортируем и получаем часть запроса для БД
		$userFilter = $filter->getFilterSql(
			$arFilter,
			explode('|', $_POST['sorter'])
		);

		/**
		 * Сортировка
		 */

		// Если сортировка не требуется, то сортировать по дате
		if (empty($_POST['sorter'])) {
			// Заглушка для фильтра, если его нет
			if (empty($userFilter)) {
				$userFilter .= ' 1=1 ';
			}

			// Часть запроса для БД
			$userFilter .= "  ORDER BY `order_date` DESC";
		}

		//Подготовка значений сортировки для вывода на страницу
		$sorterData = explode('|', $_POST['sorter']);

		if ($sorterData[1] > 0) {
			$sorterData[3] = 'desc';
		} else {
			$sorterData[3] = 'asc';
		}

		/**
		 * Страницы для пагинации
		 */

		// Определяем какую страницу вывести
		$page = !empty($_POST["page"]) ? $_POST["page"] : 0;

		// Получаем количество записей на страницу из опций
		$countPrintRows =
			MG::getSetting('countPrintRows') != null
				? MG::getSetting('countPrintRows')
				: 10;

		/**
		 *  Сборка запроса фильтрации для БД
		 */

		$sql =
			"SELECT * 
				FROM `" .
			PREFIX .
			self::$pluginName .
			"`
				WHERE " .
			$userFilter .
			"
				";

		/**
		 *  Результат
		 */

		// Получаем навигатор, который будет запрашивать записи на основе получившися правил
		// пагинации, на основе полученного запроса
		$navigator = new Navigator($sql, $page, $countPrintRows);

		// Сохраняем полученные записи для вывода на странице настроек плагина
		$entity = $navigator->getRowsSql();

		// Сохраняем форму пагинации для вывода на странице настроек плагина
		$pagination = $navigator->getPager('forAjax');

		// Сохраняем форму для фильтра для вывода на странице настроек плагина
		$filter = $filter->getHtmlFilterAdmin();

		// Способы оплаты
		$payments = Models_Order::getListPayment();

		// Способы доставки
		$deliveries = Models_Order::getListDelivery();

		// Статусы заказа
		$statuses = Models_Order::$status;

		// Организации
		$organizations = Bifit::__getOrgsData($options);

		// Торговые объекты
		$trade_objects = Bifit::__getOrgsData($options, 1);

		/**
		 *  Вывод страницы
		 */

		// Перед выводом страницы подключаем необходимые файлы css и js файлы
		self::preparePageSettings($options);

		// Вывод страницы
		include 'pageplugin.php';
	}

	/**
	 * Метод выполняющийся перед генераццией страницы настроек плагина
	 * Подключает css и js файлы плагина
	 */
	private function preparePageSettings($options)
	{
		// Подключаем файл со стилями и скрипт для страницы настроек плагина
		$css_ver = self::createFileVersion(
			dirname(__FILE__) . '/css/admin.min.css'
		);
		$js_ver = self::createFileVersion(
			dirname(__FILE__) . '/js/script.min.js'
		);
		echo '<link rel="stylesheet" href="' .
			SITE .
			'/' .
			self::$path .
			'/css/admin.css' .
			$css_ver .
			'" type="text/css" />
			<link rel="stylesheet" href="' .
			SITE .
			'/' .
			self::$path .
			'/css/timepicker.min.css" type="text/css" /> 
			<script>
			includeJS("' .
			SITE .
			'/' .
			self::$path .
			'/js/jquery-ui-timepicker-addon.js");
					includeJS("' .
			SITE .
			'/' .
			self::$path .
			'/js/script.min.js' .
			$js_ver .
			'"); 
			</script>';
	}

	/**
	 * Метод для поиска максимальной даты из всех записей в БД
	 * @return string
	 */
	private function maxDate()
	{
		$sql =
			"SELECT MAX(`order_date`) as res FROM `" .
			PREFIX .
			self::$pluginName .
			"`";
		$result = DB::fetchAssoc(DB::query($sql));

		if (!$result) {
			return false;
		}

		return $result['res'];
	}

	/**
	 * Метод для поиска минимальной даты из всех записей в БД
	 * @return string
	 */
	private function minDate()
	{
		$sql =
			"SELECT MIN(order_date) as res FROM `" .
			PREFIX .
			self::$pluginName .
			"`";
		$result = DB::fetchAssoc(DB::query($sql));

		if (!$result) {
			return false;
		}

		return $result['res'];
	}

	/**
	 * Метод для получения всех "видимых" записей
	 * @return array
	 */
	static function getAll()
	{
		// Составляем запрос на записи, которые не помечены на неотображение, и сортируем по дате
		$sql =
			"SELECT * 
				FROM `" .
			PREFIX .
			self::$pluginName .
			"` 
				ORDER BY `order_date` asc";

		// Выполняем запрос
		$result = DB::query($sql);

		// Если запрос не выполнен, вернуть false
		if (!$result) {
			return false;
		}

		// Если запрос выполнен, пройтись по результату и добавить все записи в массив на вывод
		$list = array();
		while ($row = DB::fetchAssoc($result)) {
			$list[] = $row;
		}

		// Вернуть массив с записями
		return $list;
	}

	/**
	 * Метод для получения опций плагина
	 * @return array
	 */
	private function getOptions()
	{
		// Получаем опции плагина в специальном формате
		$option = MG::getSetting(self::$pluginName . '-option');

		// Преобразуем в массив
		$option = stripslashes($option);
		$options = unserialize($option);

		// Возвращаем опции в виде получившегося массива
		return $options;
	}
}
